@@ -2,15 +2,13 @@
 
 set -ex
 
-./.gitlab-ci/check-missing-install-tag.py _build
-
-meson test \
+meson test -v \
         -C _build \
         --timeout-multiplier "${MESON_TEST_TIMEOUT_MULTIPLIER}" \
         "$@"
 
 # Run only the flaky tests, so we can log the failures but without hard failing
-meson test \
+meson test -v \
         -C _build \
         --timeout-multiplier "${MESON_TEST_TIMEOUT_MULTIPLIER}" \
         "$@" --setup=unstable_tests --suite=failing --suite=flaky || true
