@@ -2,10 +2,12 @@
  *
  * Copyright (C) 2006-2007 Red Hat, Inc.
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,20 +15,18 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Alexander Larsson <alexl@redhat.com>
  */
 
+#ifndef __G_CANCELLABLE_H__
+#define __G_CANCELLABLE_H__
+
 #if !defined (__GIO_GIO_H_INSIDE__) && !defined (GIO_COMPILATION)
 #error "Only <gio/gio.h> can be included directly."
 #endif
 
-#ifndef __G_CANCELLABLE_H__
-#define __G_CANCELLABLE_H__
-
 #include <gio/giotypes.h>
 
 G_BEGIN_DECLS
@@ -38,11 +38,6 @@ G_BEGIN_DECLS
 #define G_IS_CANCELLABLE_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), G_TYPE_CANCELLABLE))
 #define G_CANCELLABLE_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), G_TYPE_CANCELLABLE, GCancellableClass))
 
-/**
- * GCancellable:
- *
- * Allows actions to be cancelled.
- */
 typedef struct _GCancellableClass   GCancellableClass;
 typedef struct _GCancellablePrivate GCancellablePrivate;
 
@@ -69,35 +64,50 @@ struct _GCancellableClass
   void (*_g_reserved5) (void);
 };
 
+GIO_AVAILABLE_IN_ALL
 GType         g_cancellable_get_type               (void) G_GNUC_CONST;
 
+GIO_AVAILABLE_IN_ALL
 GCancellable *g_cancellable_new                    (void);
 
 /* These are only safe to call inside a cancellable op */
+GIO_AVAILABLE_IN_ALL
 gboolean      g_cancellable_is_cancelled           (GCancellable  *cancellable);
+GIO_AVAILABLE_IN_ALL
 gboolean      g_cancellable_set_error_if_cancelled (GCancellable  *cancellable,
 						    GError       **error);
 
+GIO_AVAILABLE_IN_ALL
 int           g_cancellable_get_fd                 (GCancellable  *cancellable);
+GIO_AVAILABLE_IN_ALL
 gboolean      g_cancellable_make_pollfd            (GCancellable  *cancellable,
 						    GPollFD       *pollfd);
+GIO_AVAILABLE_IN_ALL
 void          g_cancellable_release_fd             (GCancellable  *cancellable);
 
+GIO_AVAILABLE_IN_ALL
 GSource *     g_cancellable_source_new             (GCancellable  *cancellable);
 
+GIO_AVAILABLE_IN_ALL
 GCancellable *g_cancellable_get_current            (void);
+GIO_AVAILABLE_IN_ALL
 void          g_cancellable_push_current           (GCancellable  *cancellable);
+GIO_AVAILABLE_IN_ALL
 void          g_cancellable_pop_current            (GCancellable  *cancellable);
+GIO_AVAILABLE_IN_ALL
 void          g_cancellable_reset                  (GCancellable  *cancellable);
+GIO_AVAILABLE_IN_ALL
 gulong        g_cancellable_connect                (GCancellable  *cancellable,
 						    GCallback      callback,
 						    gpointer       data,
 						    GDestroyNotify data_destroy_func);
+GIO_AVAILABLE_IN_ALL
 void          g_cancellable_disconnect             (GCancellable  *cancellable,
 						    gulong         handler_id);
 
 
 /* This is safe to call from another thread */
+GIO_AVAILABLE_IN_ALL
 void          g_cancellable_cancel       (GCancellable  *cancellable);
 
 G_END_DECLS
