@@ -62,10 +62,7 @@
 #include "glibintl.h"
 
 /**
- * SECTION:gdbusaddress
- * @title: D-Bus Addresses
- * @short_description: D-Bus connection endpoints
- * @include: gio/gio.h
+ * GDBusAddress:
  *
  * Routines for working with D-Bus addresses. A D-Bus address is a string
  * like `unix:tmpdir=/tmp/my-app-name`. The exact format of addresses
@@ -622,8 +619,12 @@ g_dbus_address_connect (const gchar   *address_entry,
           g_set_error (error,
                        G_IO_ERROR,
                        G_IO_ERROR_INVALID_ARGUMENT,
-                       _("Error in address “%s” — the host attribute is missing or malformed"),
-                       address_entry);
+                       /* Translators: The first placeholder is a D-Bus connection address,
+                        * the second is the literal name of an attribute in the address.
+                        */
+                       _("Error in address “%s” — the %s attribute is missing or malformed"),
+                       address_entry,
+                       "host");
           goto out;
         }
 
@@ -636,8 +637,9 @@ g_dbus_address_connect (const gchar   *address_entry,
           g_set_error (error,
                        G_IO_ERROR,
                        G_IO_ERROR_INVALID_ARGUMENT,
-                       _("Error in address “%s” — the port attribute is missing or malformed"),
-                       address_entry);
+                       _("Error in address “%s” — the %s attribute is missing or malformed"),
+                       address_entry,
+                       "port");
           goto out;
         }
 
@@ -650,8 +652,9 @@ g_dbus_address_connect (const gchar   *address_entry,
               g_set_error (error,
                            G_IO_ERROR,
                            G_IO_ERROR_INVALID_ARGUMENT,
-                           _("Error in address “%s” — the noncefile attribute is missing or malformed"),
-                           address_entry);
+                           _("Error in address “%s” — the %s attribute is missing or malformed"),
+                           address_entry,
+                           "noncefile");
               goto out;
             }
         }
