@@ -43,85 +43,88 @@
 #include "gmarshal-internal.h"
 
 /**
- * SECTION:gdbusobjectmanagerclient
- * @short_description: Client-side object manager
- * @include: gio/gio.h
+ * GDBusObjectManagerClient:
  *
- * #GDBusObjectManagerClient is used to create, monitor and delete object
- * proxies for remote objects exported by a #GDBusObjectManagerServer (or any
- * code implementing the
+ * `GDBusObjectManagerClient` is used to create, monitor and delete object
+ * proxies for remote objects exported by a [class@Gio.DBusObjectManagerServer]
+ * (or any code implementing the
  * [org.freedesktop.DBus.ObjectManager](http://dbus.freedesktop.org/doc/dbus-specification.html#standard-interfaces-objectmanager)
  * interface).
  *
  * Once an instance of this type has been created, you can connect to
- * the #GDBusObjectManager::object-added and
- * #GDBusObjectManager::object-removed signals and inspect the
- * #GDBusObjectProxy objects returned by
- * g_dbus_object_manager_get_objects().
+ * the [signal@Gio.DBusObjectManager::object-added] and
+ * [signal@Gio.DBusObjectManager::object-removed signals] and inspect the
+ * [class@Gio.DBusObjectProxy] objects returned by
+ * [method@Gio.DBusObjectManager.get_objects].
  *
- * If the name for a #GDBusObjectManagerClient is not owned by anyone at
+ * If the name for a `GDBusObjectManagerClient` is not owned by anyone at
  * object construction time, the default behavior is to request the
  * message bus to launch an owner for the name. This behavior can be
- * disabled using the %G_DBUS_OBJECT_MANAGER_CLIENT_FLAGS_DO_NOT_AUTO_START
- * flag. It's also worth noting that this only works if the name of
+ * disabled using the `G_DBUS_OBJECT_MANAGER_CLIENT_FLAGS_DO_NOT_AUTO_START`
+ * flag. It’s also worth noting that this only works if the name of
  * interest is activatable in the first place. E.g. in some cases it
  * is not possible to launch an owner for the requested name. In this
- * case, #GDBusObjectManagerClient object construction still succeeds but
+ * case, `GDBusObjectManagerClient` object construction still succeeds but
  * there will be no object proxies
- * (e.g. g_dbus_object_manager_get_objects() returns the empty list) and
- * the #GDBusObjectManagerClient:name-owner property is %NULL.
+ * (e.g. [method@Gio.DBusObjectManager.get_objects] returns the empty list) and
+ * the [property@Gio.DBusObjectManagerClient:name-owner] property is `NULL`.
  *
  * The owner of the requested name can come and go (for example
- * consider a system service being restarted) – #GDBusObjectManagerClient
- * handles this case too; simply connect to the #GObject::notify
- * signal to watch for changes on the #GDBusObjectManagerClient:name-owner
- * property. When the name owner vanishes, the behavior is that
- * #GDBusObjectManagerClient:name-owner is set to %NULL (this includes
- * emission of the #GObject::notify signal) and then
- * #GDBusObjectManager::object-removed signals are synthesized
+ * consider a system service being restarted) – `GDBusObjectManagerClient`
+ * handles this case too; simply connect to the [signal@GObject.Object::notify]
+ * signal to watch for changes on the
+ * [property@Gio.DBusObjectManagerClient:name-owner] property. When the name
+ * owner vanishes, the behavior is that
+ * [property@Gio.DBusObjectManagerClient:name-owner] is set to `NULL` (this
+ * includes emission of the [signal@GObject.Object::notify] signal) and then
+ * [signal@Gio.DBusObjectManager::object-removed] signals are synthesized
  * for all currently existing object proxies. Since
- * #GDBusObjectManagerClient:name-owner is %NULL when this happens, you can
- * use this information to disambiguate a synthesized signal from a
- * genuine signal caused by object removal on the remote
- * #GDBusObjectManager. Similarly, when a new name owner appears,
- * #GDBusObjectManager::object-added signals are synthesized
- * while #GDBusObjectManagerClient:name-owner is still %NULL. Only when all
- * object proxies have been added, the #GDBusObjectManagerClient:name-owner
- * is set to the new name owner (this includes emission of the
- * #GObject::notify signal).  Furthermore, you are guaranteed that
- * #GDBusObjectManagerClient:name-owner will alternate between a name owner
- * (e.g. `:1.42`) and %NULL even in the case where
+ * [property@Gio.DBusObjectManagerClient:name-owner] is `NULL` when this
+ * happens, you can use this information to disambiguate a synthesized signal
+ * from a genuine signal caused by object removal on the remote
+ * [iface@Gio.DBusObjectManager]. Similarly, when a new name owner appears,
+ * [signal@Gio.DBusObjectManager::object-added] signals are synthesized
+ * while [property@Gio.DBusObjectManagerClient:name-owner] is still `NULL`. Only
+ * when all object proxies have been added, the
+ * [property@Gio.DBusObjectManagerClient:name-owner] is set to the new name
+ * owner (this includes emission of the [signal@GObject.Object::notify] signal).
+ * Furthermore, you are guaranteed that
+ * [property@Gio.DBusObjectManagerClient:name-owner] will alternate between a
+ * name owner (e.g. `:1.42`) and `NULL` even in the case where
  * the name of interest is atomically replaced
  *
- * Ultimately, #GDBusObjectManagerClient is used to obtain #GDBusProxy
- * instances. All signals (including the
- * org.freedesktop.DBus.Properties::PropertiesChanged signal)
- * delivered to #GDBusProxy instances are guaranteed to originate
+ * Ultimately, `GDBusObjectManagerClient` is used to obtain
+ * [class@Gio.DBusProxy] instances. All signals (including the
+ * `org.freedesktop.DBus.Properties::PropertiesChanged` signal)
+ * delivered to [class@Gio.DBusProxy] instances are guaranteed to originate
  * from the name owner. This guarantee along with the behavior
  * described above, means that certain race conditions including the
- * "half the proxy is from the old owner and the other half is from
- * the new owner" problem cannot happen.
+ * “half the proxy is from the old owner and the other half is from
+ * the new owner” problem cannot happen.
  *
  * To avoid having the application connect to signals on the returned
- * #GDBusObjectProxy and #GDBusProxy objects, the
- * #GDBusObject::interface-added,
- * #GDBusObject::interface-removed,
- * #GDBusProxy::g-properties-changed and
- * #GDBusProxy::g-signal signals
- * are also emitted on the #GDBusObjectManagerClient instance managing these
+ * [class@Gio.DBusObjectProxy] and [class@Gio.DBusProxy] objects, the
+ * [signal@Gio.DBusObject::interface-added],
+ * [signal@Gio.DBusObject::interface-removed],
+ * [signal@Gio.DBusProxy::g-properties-changed] and
+ * [signal@Gio.DBusProxy::g-signal] signals
+ * are also emitted on the `GDBusObjectManagerClient` instance managing these
  * objects. The signals emitted are
- * #GDBusObjectManager::interface-added,
- * #GDBusObjectManager::interface-removed,
- * #GDBusObjectManagerClient::interface-proxy-properties-changed and
- * #GDBusObjectManagerClient::interface-proxy-signal.
+ * [signal@Gio.DBusObjectManager::interface-added],
+ * [signal@Gio.DBusObjectManager::interface-removed],
+ * [signal@Gio.DBusObjectManagerClient::interface-proxy-properties-changed] and
+ * [signal@Gio.DBusObjectManagerClient::interface-proxy-signal].
  *
  * Note that all callbacks and signals are emitted in the
- * [thread-default main context][g-main-context-push-thread-default]
- * that the #GDBusObjectManagerClient object was constructed
- * in. Additionally, the #GDBusObjectProxy and #GDBusProxy objects
- * originating from the #GDBusObjectManagerClient object will be created in
+ * thread-default main context (see
+ * [method@GLib.MainContext.push_thread_default]) that the
+ * `GDBusObjectManagerClient` object was constructed in. Additionally, the
+ * [class@Gio.DBusObjectProxy] and [class@Gio.DBusProxy] objects
+ * originating from the `GDBusObjectManagerClient` object will be created in
  * the same context and, consequently, will deliver signals in the
  * same main loop.
+ *
+ * Since: 2.30
  */
 
 struct _GDBusObjectManagerClientPrivate
@@ -360,9 +363,7 @@ g_dbus_object_manager_client_class_init (GDBusObjectManagerClientClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_CONNECTION,
-                                   g_param_spec_object ("connection",
-                                                        "Connection",
-                                                        "The connection to use",
+                                   g_param_spec_object ("connection", NULL, NULL,
                                                         G_TYPE_DBUS_CONNECTION,
                                                         G_PARAM_READABLE |
                                                         G_PARAM_WRITABLE |
@@ -381,9 +382,7 @@ g_dbus_object_manager_client_class_init (GDBusObjectManagerClientClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_BUS_TYPE,
-                                   g_param_spec_enum ("bus-type",
-                                                      "Bus Type",
-                                                      "The bus to connect to, if any",
+                                   g_param_spec_enum ("bus-type", NULL, NULL,
                                                       G_TYPE_BUS_TYPE,
                                                       G_BUS_TYPE_NONE,
                                                       G_PARAM_WRITABLE |
@@ -401,9 +400,7 @@ g_dbus_object_manager_client_class_init (GDBusObjectManagerClientClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_FLAGS,
-                                   g_param_spec_flags ("flags",
-                                                       "Flags",
-                                                       "Flags for the proxy manager",
+                                   g_param_spec_flags ("flags", NULL, NULL,
                                                        G_TYPE_DBUS_OBJECT_MANAGER_CLIENT_FLAGS,
                                                        G_DBUS_OBJECT_MANAGER_CLIENT_FLAGS_NONE,
                                                        G_PARAM_READABLE |
@@ -422,9 +419,7 @@ g_dbus_object_manager_client_class_init (GDBusObjectManagerClientClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_OBJECT_PATH,
-                                   g_param_spec_string ("object-path",
-                                                        "Object Path",
-                                                        "The object path of the control object",
+                                   g_param_spec_string ("object-path", NULL, NULL,
                                                         NULL,
                                                         G_PARAM_READABLE |
                                                         G_PARAM_WRITABLE |
@@ -440,9 +435,7 @@ g_dbus_object_manager_client_class_init (GDBusObjectManagerClientClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_NAME,
-                                   g_param_spec_string ("name",
-                                                        "Name",
-                                                        "Name that the manager is for",
+                                   g_param_spec_string ("name", NULL, NULL,
                                                         NULL,
                                                         G_PARAM_READABLE |
                                                         G_PARAM_WRITABLE |
@@ -460,9 +453,7 @@ g_dbus_object_manager_client_class_init (GDBusObjectManagerClientClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_NAME_OWNER,
-                                   g_param_spec_string ("name-owner",
-                                                        "Name Owner",
-                                                        "The owner of the name we are watching",
+                                   g_param_spec_string ("name-owner", NULL, NULL,
                                                         NULL,
                                                         G_PARAM_READABLE |
                                                         G_PARAM_STATIC_STRINGS));
@@ -477,9 +468,7 @@ g_dbus_object_manager_client_class_init (GDBusObjectManagerClientClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_GET_PROXY_TYPE_FUNC,
-                                   g_param_spec_pointer ("get-proxy-type-func",
-                                                         "GDBusProxyTypeFunc Function Pointer",
-                                                         "The GDBusProxyTypeFunc pointer to use",
+                                   g_param_spec_pointer ("get-proxy-type-func", NULL, NULL,
                                                          G_PARAM_READABLE |
                                                          G_PARAM_WRITABLE |
                                                          G_PARAM_CONSTRUCT_ONLY |
@@ -494,9 +483,7 @@ g_dbus_object_manager_client_class_init (GDBusObjectManagerClientClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_GET_PROXY_TYPE_USER_DATA,
-                                   g_param_spec_pointer ("get-proxy-type-user-data",
-                                                         "GDBusProxyTypeFunc User Data",
-                                                         "The GDBusProxyTypeFunc user_data",
+                                   g_param_spec_pointer ("get-proxy-type-user-data", NULL, NULL,
                                                          G_PARAM_READABLE |
                                                          G_PARAM_WRITABLE |
                                                          G_PARAM_CONSTRUCT_ONLY |
@@ -511,9 +498,7 @@ g_dbus_object_manager_client_class_init (GDBusObjectManagerClientClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_GET_PROXY_TYPE_DESTROY_NOTIFY,
-                                   g_param_spec_pointer ("get-proxy-type-destroy-notify",
-                                                         "GDBusProxyTypeFunc user data free function",
-                                                         "The GDBusProxyTypeFunc user data free function",
+                                   g_param_spec_pointer ("get-proxy-type-destroy-notify", NULL, NULL,
                                                          G_PARAM_READABLE |
                                                          G_PARAM_WRITABLE |
                                                          G_PARAM_CONSTRUCT_ONLY |
@@ -533,8 +518,8 @@ g_dbus_object_manager_client_class_init (GDBusObjectManagerClientClass *klass)
    * This signal exists purely as a convenience to avoid having to
    * connect signals to all interface proxies managed by @manager.
    *
-   * This signal is emitted in the
-   * [thread-default main context][g-main-context-push-thread-default]
+   * This signal is emitted in the thread-default main context
+   * (see [method@GLib.MainContext.push_thread_default])
    * that @manager was constructed in.
    *
    * Since: 2.30
@@ -575,8 +560,8 @@ g_dbus_object_manager_client_class_init (GDBusObjectManagerClientClass *klass)
    * This signal exists purely as a convenience to avoid having to
    * connect signals to all interface proxies managed by @manager.
    *
-   * This signal is emitted in the
-   * [thread-default main context][g-main-context-push-thread-default]
+   * This signal is emitted in the thread-default main context
+   * (see [method@GLib.MainContext.push_thread_default])
    * that @manager was constructed in.
    *
    * Since: 2.30
@@ -690,10 +675,10 @@ g_dbus_object_manager_client_new_sync (GDBusConnection               *connection
  * Asynchronously creates a new #GDBusObjectManagerClient object.
  *
  * This is an asynchronous failable constructor. When the result is
- * ready, @callback will be invoked in the
- * [thread-default main context][g-main-context-push-thread-default]
- * of the thread you are calling this method from. You can
- * then call g_dbus_object_manager_client_new_finish() to get the result. See
+ * ready, @callback will be invoked in the thread-default main context
+ * (see [method@GLib.MainContext.push_thread_default])
+ * of the thread you are calling this method from. You can then call
+ * g_dbus_object_manager_client_new_finish() to get the result. See
  * g_dbus_object_manager_client_new_sync() for the synchronous version.
  *
  * Since: 2.30
@@ -843,8 +828,8 @@ g_dbus_object_manager_client_new_for_bus_sync (GBusType                       bu
  * #GDBusConnection.
  *
  * This is an asynchronous failable constructor. When the result is
- * ready, @callback will be invoked in the
- * [thread-default main loop][g-main-context-push-thread-default]
+ * ready, @callback will be invoked in the thread-default main context
+ * (see [method@GLib.MainContext.push_thread_default])
  * of the thread you are calling this method from. You can
  * then call g_dbus_object_manager_client_new_for_bus_finish() to get the result. See
  * g_dbus_object_manager_client_new_for_bus_sync() for the synchronous version.
@@ -1041,7 +1026,7 @@ signal_cb (GDBusConnection *connection,
   //g_debug ("yay, signal_cb %s %s: %s\n", signal_name, object_path, g_variant_print (parameters, TRUE));
 
   g_object_ref (manager);
-  if (g_strcmp0 (interface_name, "org.freedesktop.DBus.Properties") == 0)
+  if (g_strcmp0 (interface_name, DBUS_INTERFACE_PROPERTIES) == 0)
     {
       if (g_strcmp0 (signal_name, "PropertiesChanged") == 0)
         {
@@ -1160,9 +1145,9 @@ subscribe_signals (GDBusObjectManagerClient *manager,
       /* The bus daemon may not implement path_namespace so gracefully
        * handle this by using a fallback triggered if @error is set. */
       ret = g_dbus_connection_call_sync (manager->priv->connection,
-                                         "org.freedesktop.DBus",
-                                         "/org/freedesktop/DBus",
-                                         "org.freedesktop.DBus",
+                                         DBUS_SERVICE_DBUS,
+                                         DBUS_PATH_DBUS,
+                                         DBUS_INTERFACE_DBUS,
                                          "AddMatch",
                                          g_variant_new ("(s)",
                                                         manager->priv->match_rule),
@@ -1233,11 +1218,8 @@ maybe_unsubscribe_signals (GDBusObjectManagerClient *manager)
   g_return_if_fail (G_IS_DBUS_OBJECT_MANAGER_CLIENT (manager));
 
   if (manager->priv->signal_subscription_id > 0)
-    {
-      g_dbus_connection_signal_unsubscribe (manager->priv->connection,
-                                            manager->priv->signal_subscription_id);
-      manager->priv->signal_subscription_id = 0;
-    }
+    g_dbus_connection_signal_unsubscribe (manager->priv->connection,
+                                          g_steal_handle_id (&manager->priv->signal_subscription_id));
 
   if (manager->priv->match_rule != NULL)
     {
@@ -1245,9 +1227,9 @@ maybe_unsubscribe_signals (GDBusObjectManagerClient *manager)
        * fail - therefore, don't bother checking the return value
        */
       g_dbus_connection_call (manager->priv->connection,
-                              "org.freedesktop.DBus",
-                              "/org/freedesktop/DBus",
-                              "org.freedesktop.DBus",
+                              DBUS_SERVICE_DBUS,
+                              DBUS_PATH_DBUS,
+                              DBUS_INTERFACE_DBUS,
                               "RemoveMatch",
                               g_variant_new ("(s)",
                                              manager->priv->match_rule),
@@ -1355,13 +1337,11 @@ on_notify_g_name_owner (GObject    *object,
 
   if (g_strcmp0 (old_name_owner, new_name_owner) != 0)
     {
-      GList *l;
-      GList *proxies;
+      GPtrArray *proxies;
 
       /* remote manager changed; nuke all local proxies  */
-      proxies = g_hash_table_get_values (manager->priv->map_object_path_to_object_proxy);
-      g_list_foreach (proxies, (GFunc) g_object_ref, NULL);
-      g_hash_table_remove_all (manager->priv->map_object_path_to_object_proxy);
+      proxies = g_hash_table_steal_all_values (
+        manager->priv->map_object_path_to_object_proxy);
 
       g_mutex_unlock (&manager->priv->lock);
 
@@ -1371,12 +1351,13 @@ on_notify_g_name_owner (GObject    *object,
        */
       g_object_notify (G_OBJECT (manager), "name-owner");
 
-      for (l = proxies; l != NULL; l = l->next)
+      for (guint i = 0; i < proxies->len; ++i)
         {
-          GDBusObjectProxy *object_proxy = G_DBUS_OBJECT_PROXY (l->data);
+          GDBusObjectProxy *object_proxy =
+            G_DBUS_OBJECT_PROXY (g_ptr_array_index (proxies, i));
           g_signal_emit_by_name (manager, "object-removed", object_proxy);
         }
-      g_list_free_full (proxies, g_object_unref);
+      g_clear_pointer (&proxies, g_ptr_array_unref);
 
       /* nuke local filter */
       maybe_unsubscribe_signals (manager);
@@ -1435,7 +1416,7 @@ initable_init (GInitable     *initable,
                                                         NULL, /* GDBusInterfaceInfo* */
                                                         manager->priv->name,
                                                         manager->priv->object_path,
-                                                        "org.freedesktop.DBus.ObjectManager",
+                                                        DBUS_INTERFACE_OBJECT_MANAGER,
                                                         cancellable,
                                                         error);
   if (manager->priv->control_proxy == NULL)
@@ -1456,7 +1437,7 @@ initable_init (GInitable     *initable,
                             G_CALLBACK (on_notify_g_name_owner),
                             weak_ref_new (G_OBJECT (manager)),
                             (GClosureNotify) weak_ref_free,
-                            0  /* flags */);
+                            G_CONNECT_DEFAULT);
 
   manager->priv->signal_signal_id =
       g_signal_connect_data (manager->priv->control_proxy,
@@ -1464,7 +1445,7 @@ initable_init (GInitable     *initable,
                             G_CALLBACK (on_control_proxy_g_signal),
                             weak_ref_new (G_OBJECT (manager)),
                             (GClosureNotify) weak_ref_free,
-                            0  /* flags */);
+                            G_CONNECT_DEFAULT);
 
   manager->priv->name_owner = g_dbus_proxy_get_name_owner (manager->priv->control_proxy);
   if (manager->priv->name_owner == NULL && manager->priv->name != NULL)
