@@ -806,7 +806,7 @@ get_mount_info (GFileInfo             *fs_info,
 					     g_free, NULL);
 
 
-  if (g_unix_mounts_changed_since (mount_info_hash_cache_time))
+  if (g_unix_mount_entries_changed_since (mount_info_hash_cache_time))
     g_hash_table_remove_all (mount_info_hash);
   
   got_info = g_hash_table_lookup_extended (mount_info_hash,
@@ -826,15 +826,15 @@ get_mount_info (GFileInfo             *fs_info,
       if (mountpoint == NULL)
 	mountpoint = g_strdup ("/");
 
-      mount = g_unix_mount_at (mountpoint, &cache_time);
+      mount = g_unix_mount_entry_at (mountpoint, &cache_time);
       if (mount)
 	{
-	  if (g_unix_mount_is_readonly (mount))
+	  if (g_unix_mount_entry_is_readonly (mount))
 	    mount_info |= MOUNT_INFO_READONLY;
-          if (is_remote_fs_type (g_unix_mount_get_fs_type (mount)))
+          if (is_remote_fs_type (g_unix_mount_entry_get_fs_type (mount)))
             is_remote = TRUE;
 	  
-	  g_unix_mount_free (mount);
+	  g_unix_mount_entry_free (mount);
 	}
 
       g_free (mountpoint);
@@ -1189,6 +1189,7 @@ g_local_file_set_display_name (GFile         *file,
       if (errsv != ENOENT)
         {
           g_set_io_error (error, _("Error renaming file %s: %s"), new_file, errsv);
+          g_object_unref (new_file);
           return NULL;
         }
     }
@@ -1196,6 +1197,7 @@ g_local_file_set_display_name (GFile         *file,
     {
       g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_EXISTS,
                            _("Can’t rename file, filename already exists"));
+      g_object_unref (new_file);
       return NULL;
     }
 
@@ -1259,6 +1261,24 @@ g_local_file_query_info (GFile                *file,
   return info;
 }
 
+/* FIXME: faccessat() is available on FreeBSD but appears to not work correctly
+ * here. This needs diagnosing; https://gitlab.gnome.org/GNOME/glib/-/issues/3495
+ *
+ * On Android (bionic as of 2015-02-24), faccess() returns EINVAL if any flags are set,
+ * so we have to use the fallback path. See
+ * https://cs.android.com/android/_/android/platform/bionic/+/35778253a5ed71e87a608ca590b63729d9f88567
+ */
+#if defined(HAVE_FACCESSAT) && !defined(__FreeBSD__) && !defined(__ANDROID__)
+static gboolean
+g_local_file_query_exists (GFile        *file,
+                           GCancellable *cancellable)
+{
+  GLocalFile *local = G_LOCAL_FILE (file);
+
+  return faccessat (0, local->filename, F_OK, AT_EACCESS | AT_SYMLINK_NOFOLLOW) == 0;
+}
+#endif
+
 static GFileAttributeInfoList *
 g_local_file_query_settable_attributes (GFile         *file,
 					GCancellable  *cancellable,
@@ -1808,10 +1828,10 @@ ignore_trash_mount (GUnixMountEntry *mount)
   GUnixMountPoint *mount_point = NULL;
   const gchar *mount_options;
 
-  mount_options = g_unix_mount_get_options (mount);
+  mount_options = g_unix_mount_entry_get_options (mount);
   if (mount_options == NULL)
     {
-      mount_point = g_unix_mount_point_at (g_unix_mount_get_mount_path (mount),
+      mount_point = g_unix_mount_point_at (g_unix_mount_entry_get_mount_path (mount),
                                            NULL);
       if (mount_point != NULL)
         mount_options = g_unix_mount_point_get_options (mount_point);
@@ -1828,7 +1848,7 @@ ignore_trash_mount (GUnixMountEntry *mount)
         return TRUE;
     }
 
-  if (g_unix_mount_is_system_internal (mount))
+  if (g_unix_mount_entry_is_system_internal (mount))
     return TRUE;
 
   return FALSE;
@@ -1840,14 +1860,14 @@ ignore_trash_path (const gchar *topdir)
   GUnixMountEntry *mount;
   gboolean retval = TRUE;
 
-  mount = g_unix_mount_at (topdir, NULL);
+  mount = g_unix_mount_entry_at (topdir, NULL);
   if (mount == NULL)
     goto out;
 
   retval = ignore_trash_mount (mount);
 
  out:
-  g_clear_pointer (&mount, g_unix_mount_free);
+  g_clear_pointer (&mount, g_unix_mount_entry_free);
 
   return retval;
 }
@@ -3142,6 +3162,9 @@ g_local_file_file_iface_init (GFileIface *iface)
   iface->monitor_dir = g_local_file_monitor_dir;
   iface->monitor_file = g_local_file_monitor_file;
   iface->measure_disk_usage = g_local_file_measure_disk_usage;
+#if defined(HAVE_FACCESSAT) && !defined(__FreeBSD__) && !defined(__ANDROID__)
+  iface->query_exists = g_local_file_query_exists;
+#endif
 
   iface->supports_thread_contexts = TRUE;
 }
