@@ -103,12 +103,6 @@
 #include "glibintl.h"
 
 
-struct ThumbMD5Context {
-	guint32 buf[4];
-	guint32 bits[2];
-	unsigned char in[64];
-};
-
 #ifndef G_OS_WIN32
 
 typedef struct {
@@ -1345,7 +1339,11 @@ get_content_type (const char          *basename,
 #if !defined(G_OS_WIN32) && !defined(__APPLE__)
       if (!fast && result_uncertain && path != NULL)
 	{
-	  guchar sniff_buffer[4096];
+          /* Sniff the first 16KiB of the file (sometimes less, if xdgmime
+           * says it doesn’t need so much). Most files need less than 4KiB of
+           * sniffing, but some disk images need more (see
+           * https://gitlab.gnome.org/GNOME/glib/-/issues/3186). */
+	  guchar sniff_buffer[16384];
 	  gsize sniff_length;
 #ifdef O_NOATIME
 	  int errsv;
@@ -1353,8 +1351,8 @@ get_content_type (const char          *basename,
 	  int fd;
 
 	  sniff_length = _g_unix_content_type_get_sniff_len ();
-	  if (sniff_length == 0 || sniff_length > 4096)
-	    sniff_length = 4096;
+	  if (sniff_length == 0 || sniff_length > sizeof (sniff_buffer))
+	    sniff_length = sizeof (sniff_buffer);
 
 #ifdef O_NOATIME	  
           fd = g_open (path, O_RDONLY | O_NOATIME | O_CLOEXEC, 0);
@@ -2023,13 +2021,14 @@ _g_local_file_info_get (const char             *basename,
   _g_file_info_set_attribute_boolean_by_id (info, G_FILE_ATTRIBUTE_ID_DOS_IS_SYSTEM,
                                             (statbuf.attributes & FILE_ATTRIBUTE_SYSTEM));
 
-  _g_file_info_set_attribute_boolean_by_id (info, G_FILE_ATTRIBUTE_ID_DOS_IS_MOUNTPOINT,
-                                            (statbuf.reparse_tag == IO_REPARSE_TAG_MOUNT_POINT));
+  if (stat_ok)
+    {
+      _g_file_info_set_attribute_boolean_by_id (info, G_FILE_ATTRIBUTE_ID_DOS_IS_MOUNTPOINT,
+                                                (statbuf.reparse_tag == IO_REPARSE_TAG_MOUNT_POINT));
 
-  if (statbuf.reparse_tag != 0)
-    _g_file_info_set_attribute_uint32_by_id (info, G_FILE_ATTRIBUTE_ID_DOS_REPARSE_POINT_TAG, statbuf.reparse_tag);
-
-  _g_file_info_set_attribute_boolean_by_id (info, G_FILE_ATTRIBUTE_ID_STANDARD_IS_BACKUP, FALSE);
+      if (statbuf.reparse_tag != 0)
+        _g_file_info_set_attribute_uint32_by_id (info, G_FILE_ATTRIBUTE_ID_DOS_REPARSE_POINT_TAG, statbuf.reparse_tag);
+    }
 #endif
 
   symlink_target = NULL;
@@ -2832,6 +2831,16 @@ set_mtime_atime (char                       *filename,
 	}
     }
 
+  if (atime_usec_value)
+    {
+      guint32 val_usec = 0;
+
+      if (!get_uint32 (atime_usec_value, &val_usec, error))
+        return FALSE;
+
+      times_n[0].tv_nsec = val_usec * 1000;
+    }
+
   if (atime_nsec_value)
     {
       guint32 val_nsec = 0;
@@ -2861,6 +2870,16 @@ set_mtime_atime (char                       *filename,
 	}
     }
 
+  if (mtime_usec_value)
+    {
+      guint32 val_usec = 0;
+
+      if (!get_uint32 (mtime_usec_value, &val_usec, error))
+        return FALSE;
+
+      times_n[1].tv_nsec = val_usec * 1000;
+    }
+
   if (mtime_nsec_value)
     {
       guint32 val_nsec = 0;
