@@ -2,6 +2,8 @@
  *
  * Copyright 2011 Red Hat, Inc.
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -283,8 +285,9 @@ can_reach_async_got_address (GObject      *object,
       else
         {
           /* Resolved all addresses, none matched */
-          g_task_return_new_error (task, G_IO_ERROR, G_IO_ERROR_HOST_UNREACHABLE,
-                                   _("Host unreachable"));
+          g_task_return_new_error_literal (task, G_IO_ERROR,
+                                           G_IO_ERROR_HOST_UNREACHABLE,
+                                           _("Host unreachable"));
           g_object_unref (task);
           return;
         }
@@ -319,8 +322,8 @@ g_network_monitor_base_can_reach_async (GNetworkMonitor     *monitor,
 
   if (g_hash_table_size (G_NETWORK_MONITOR_BASE (monitor)->priv->networks) == 0)
     {
-      g_task_return_new_error (task, G_IO_ERROR, G_IO_ERROR_NETWORK_UNREACHABLE,
-                               _("Network unreachable"));
+      g_task_return_new_error_literal (task, G_IO_ERROR, G_IO_ERROR_NETWORK_UNREACHABLE,
+                                       _("Network unreachable"));
       g_object_unref (task);
       return;
     }
@@ -467,7 +470,7 @@ queue_network_changed (GNetworkMonitorBase *monitor)
        */
       g_source_set_priority (source, G_PRIORITY_HIGH_IDLE);
       g_source_set_callback (source, emit_network_changed, monitor, NULL);
-      g_source_set_name (source, "[gio] emit_network_changed");
+      g_source_set_static_name (source, "[gio] emit_network_changed");
       g_source_attach (source, monitor->priv->context);
       monitor->priv->network_changed_source = source;
     }
