@@ -4,10 +4,12 @@
  *
  * Copyright (C) 2008 Red Hat, Inc.
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,24 +31,24 @@
 
 
 /**
- * SECTION:gsrvtarget
- * @short_description: DNS SRV record target
- * @include: gio/gio.h
+ * GSrvTarget:
+ *
+ * A single target host/port that a network service is running on.
  *
  * SRV (service) records are used by some network protocols to provide
  * service-specific aliasing and load-balancing. For example, XMPP
  * (Jabber) uses SRV records to locate the XMPP server for a domain;
- * rather than connecting directly to "example.com" or assuming a
- * specific server hostname like "xmpp.example.com", an XMPP client
- * would look up the "xmpp-client" SRV record for "example.com", and
+ * rather than connecting directly to ‘example.com’ or assuming a
+ * specific server hostname like ‘xmpp.example.com’, an XMPP client
+ * would look up the `xmpp-client` SRV record for ‘example.com’, and
  * then connect to whatever host was pointed to by that record.
  *
- * You can use g_resolver_lookup_service() or
- * g_resolver_lookup_service_async() to find the #GSrvTargets
+ * You can use [method@Gio.Resolver.lookup_service] or
+ * [method@Gio.Resolver.lookup_service_async] to find the `GSrvTarget`s
  * for a given service. However, if you are simply planning to connect
- * to the remote service, you can use #GNetworkService's
- * #GSocketConnectable interface and not need to worry about
- * #GSrvTarget at all.
+ * to the remote service, you can use [class@Gio.NetworkService]’s
+ * [iface@Gio.SocketConnectable] interface and not need to worry about
+ * `GSrvTarget` at all.
  */
 
 struct _GSrvTarget {
@@ -57,12 +59,6 @@ struct _GSrvTarget {
   guint16  weight;
 };
 
-/**
- * GSrvTarget:
- *
- * A single target host/port that a network service is running on.
- */
-
 G_DEFINE_BOXED_TYPE (GSrvTarget, g_srv_target,
                      g_srv_target_copy, g_srv_target_free)
 
@@ -289,6 +285,7 @@ g_srv_target_list_sort (GList *targets)
           val = g_random_int_range (0, sum + 1);
           for (t = targets; ; t = t->next)
             {
+              g_assert (t != NULL && t->data != NULL);
               weight = ((GSrvTarget *)t->data)->weight;
               if (weight >= val)
                 break;
