@@ -2,6 +2,8 @@
  *
  * Copyright © 2009 Codethink Limited
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -12,35 +14,27 @@
  * Authors: Ryan Lortie <desrt@desrt.ca>
  */
 
-/**
- * SECTION:gunixfdlist
- * @title: GUnixFDList
- * @short_description: An object containing a set of UNIX file descriptors
- * @include: gio/gunixfdlist.h
- * @see_also: #GUnixFDMessage
- *
- * A #GUnixFDList contains a list of file descriptors.  It owns the file
- * descriptors that it contains, closing them when finalized.
- *
- * It may be wrapped in a #GUnixFDMessage and sent over a #GSocket in
- * the %G_SOCKET_FAMILY_UNIX family by using g_socket_send_message()
- * and received using g_socket_receive_message().
- *
- * Note that `<gio/gunixfdlist.h>` belongs to the UNIX-specific GIO
- * interfaces, thus you have to use the `gio-unix-2.0.pc` pkg-config
- * file when using it.
- */
-
 /**
  * GUnixFDList:
  *
- * #GUnixFDList is an opaque data structure and can only be accessed
- * using the following functions.
- **/
+ * A `GUnixFDList` contains a list of file descriptors.  It owns the file
+ * descriptors that it contains, closing them when finalized.
+ *
+ * It may be wrapped in a
+ * [`GUnixFDMessage`](../gio-unix/class.UnixFDMessage.html) and sent over a
+ * [class@Gio.Socket] in the `G_SOCKET_FAMILY_UNIX` family by using
+ * [method@Gio.Socket.send_message] and received using
+ * [method@Gio.Socket.receive_message].
+ *
+ * Before 2.74, `<gio/gunixfdlist.h>` belonged to the UNIX-specific GIO
+ * interfaces, thus you had to use the `gio-unix-2.0.pc` pkg-config file when
+ * using it.
+ *
+ * Since 2.74, the API is available for Windows.
+ */
 
 #include "config.h"
 
-#include <unistd.h>
 #include <fcntl.h>
 #include <string.h>
 #include <errno.h>
@@ -48,6 +42,12 @@
 #include "gunixfdlist.h"
 #include "gnetworking.h"
 #include "gioerror.h"
+#include "glib/glib-private.h"
+#include "glib/gstdio.h"
+
+#ifdef G_OS_WIN32
+#include <io.h>
+#endif
 
 struct _GUnixFDListPrivate
 {
@@ -70,7 +70,7 @@ g_unix_fd_list_finalize (GObject *object)
   gint i;
 
   for (i = 0; i < list->priv->nfd; i++)
-    close (list->priv->fds[i]);
+    g_close (list->priv->fds[i], NULL);
   g_free (list->priv->fds);
 
   G_OBJECT_CLASS (g_unix_fd_list_parent_class)
@@ -90,7 +90,9 @@ dup_close_on_exec_fd (gint     fd,
                       GError **error)
 {
   gint new_fd;
+#ifndef G_OS_WIN32
   gint s;
+#endif
 
 #ifdef F_DUPFD_CLOEXEC
   do
@@ -118,6 +120,9 @@ dup_close_on_exec_fd (gint     fd,
       return -1;
     }
 
+#ifdef G_OS_WIN32
+  new_fd = GLIB_PRIVATE_CALL (g_win32_reopen_noninherited) (new_fd, 0, error);
+#else
   do
     {
       s = fcntl (new_fd, F_GETFD);
@@ -134,10 +139,11 @@ dup_close_on_exec_fd (gint     fd,
       g_set_error (error, G_IO_ERROR,
                    g_io_error_from_errno (saved_errno),
                    "fcntl: %s", g_strerror (saved_errno));
-      close (new_fd);
+      g_close (new_fd, NULL);
 
       return -1;
     }
+#endif
 
   return new_fd;
 }
