@@ -54,10 +54,10 @@ static void
 startup (GApplication *app)
 {
   static GActionEntry actions[] = {
-    { "new", new_activated, NULL, NULL, NULL },
-    { "quit", quit_activated, NULL, NULL, NULL },
-    { "action1", action1_activated, NULL, NULL, NULL },
-    { "action2", action2_activated, "b", "false", change_action2 }
+    { "new", new_activated, NULL, NULL, NULL, { 0 } },
+    { "quit", quit_activated, NULL, NULL, NULL, { 0 } },
+    { "action1", action1_activated, NULL, NULL, NULL, { 0 } },
+    { "action2", action2_activated, "b", "false", change_action2, { 0 } }
   };
 
   g_action_map_add_action_entries (G_ACTION_MAP (app),
@@ -74,10 +74,10 @@ activate (GApplication *application)
 }
 
 static void
-open (GApplication  *application,
-      GFile        **files,
-      gint           n_files,
-      const gchar   *hint)
+app_open (GApplication  *application,
+          GFile        **files,
+          gint           n_files,
+          const gchar   *hint)
 {
   gint i;
 
@@ -249,7 +249,7 @@ main (int argc, char **argv)
                              : G_APPLICATION_HANDLES_OPEN));
   g_signal_connect (app, "startup", G_CALLBACK (startup), NULL);
   g_signal_connect (app, "activate", G_CALLBACK (activate), NULL);
-  g_signal_connect (app, "open", G_CALLBACK (open), NULL);
+  g_signal_connect (app, "open", G_CALLBACK (app_open), NULL);
   g_signal_connect (app, "command-line", G_CALLBACK (command_line), NULL);
 #ifdef STANDALONE
   g_application_set_inactivity_timeout (app, 10000);
