@@ -837,7 +837,7 @@ cancellable_thread_cb (gpointer data)
 {
   GCancellable *cancellable = data;
 
-  g_usleep (0.1 * G_USEC_PER_SEC);
+  g_usleep ((guint64) (0.1 * G_USEC_PER_SEC));
   g_cancellable_cancel (cancellable);
   g_object_unref (cancellable);
 
@@ -2537,6 +2537,7 @@ test_receive_bytes_from (void)
   g_socket_close (data->server, &error);
   g_assert_no_error (error);
 
+  g_object_unref (dest_addr);
   g_object_unref (client);
 
   ip_test_data_free (data);
