@@ -732,7 +732,8 @@ g_system_thread_new (GThreadFunc proxy,
   base_thread->thread.joinable = TRUE;
   base_thread->thread.func = func;
   base_thread->thread.data = data;
-  base_thread->name = g_strdup (name);
+  if (name)
+    g_strlcpy (base_thread->name, name, sizeof (base_thread->name));
   thread->proxy = proxy;
 
   posix_check_cmd (pthread_attr_init (&attr));
@@ -766,7 +767,6 @@ g_system_thread_new (GThreadFunc proxy,
     {
       g_set_error (error, G_THREAD_ERROR, G_THREAD_ERROR_AGAIN, 
                    "Error creating thread: %s", g_strerror (ret));
-      g_free (thread->thread.name);
       g_slice_free (GThreadPosix, thread);
       return NULL;
     }
@@ -827,6 +827,18 @@ g_system_thread_set_name (const gchar *name)
 #endif
 }
 
+void
+g_system_thread_get_name (char  *buffer,
+                          gsize  length)
+{
+#ifdef HAVE_PTHREAD_GETNAME_NP
+  pthread_getname_np (pthread_self (), buffer, length);
+#else
+  g_assert (length >= 1);
+  buffer[0] = '\0';
+#endif
+}
+
 /* {{{1 GMutex and GCond futex implementation */
 
 #if defined(USE_NATIVE_MUTEX)
