@@ -60,6 +60,8 @@
 #define G_TEST_DIR_MODE (S_IWRITE | S_IREAD)
 #endif
 
+#include "testutils.h"
+
 #define S G_DIR_SEPARATOR_S
 
 static void
@@ -662,7 +664,7 @@ test_build_filenamev (void)
 static void
 test_mkdir_with_parents_1 (const gchar *base)
 {
-  char *p0 = g_build_filename (base, "fum", NULL);
+  char *p0 = g_build_filename (g_get_tmp_dir (), base, "fum", NULL);
   char *p1 = g_build_filename (p0, "tem", NULL);
   char *p2 = g_build_filename (p1, "zap", NULL);
   FILE *f;
@@ -815,21 +817,18 @@ test_mkdir_with_parents (void)
 #ifndef G_OS_WIN32
   gboolean can_override_dac = check_cap_dac_override (NULL);
 #endif
-  if (g_test_verbose())
-    g_printerr ("checking g_mkdir_with_parents() in subdir ./hum/");
+
+  g_test_message ("Checking g_mkdir_with_parents() in subdir ./hum/");
   test_mkdir_with_parents_1 ("hum");
   g_remove ("hum");
-  if (g_test_verbose())
-    g_printerr ("checking g_mkdir_with_parents() in subdir ./hii///haa/hee/");
+
+  g_test_message ("Checking g_mkdir_with_parents() in subdir ./hii///haa/hee/");
   test_mkdir_with_parents_1 ("./hii///haa/hee///");
   g_remove ("hii/haa/hee");
   g_remove ("hii/haa");
   g_remove ("hii");
-  cwd = g_get_current_dir ();
-  if (g_test_verbose())
-    g_printerr ("checking g_mkdir_with_parents() in cwd: %s", cwd);
-  test_mkdir_with_parents_1 (cwd);
 
+  cwd = g_get_current_dir ();
   new_path = g_build_filename (cwd, "new", NULL);
   g_assert_cmpint (g_mkdir_with_parents (new_path, 0), ==, 0);
   g_assert_cmpint (g_rmdir (new_path), ==, 0);
@@ -979,17 +978,17 @@ test_format_size_for_display (void)
   check_string (g_format_size_full (2, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_ONLY_VALUE), "2");
   check_string (g_format_size_full (2, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_ONLY_UNIT), "bits");
 
-  check_string (g_format_size_full (2000ULL, G_FORMAT_SIZE_BITS), "2.0\302\240kb");
-  check_string (g_format_size_full (2000ULL * 1000, G_FORMAT_SIZE_BITS), "2.0\302\240Mb");
-  check_string (g_format_size_full (2000ULL * 1000 * 1000, G_FORMAT_SIZE_BITS), "2.0\302\240Gb");
-  check_string (g_format_size_full (2000ULL * 1000 * 1000 * 1000, G_FORMAT_SIZE_BITS), "2.0\302\240Tb");
-  check_string (g_format_size_full (2000ULL * 1000 * 1000 * 1000 * 1000, G_FORMAT_SIZE_BITS), "2.0\302\240Pb");
-  check_string (g_format_size_full (2000ULL * 1000 * 1000 * 1000 * 1000 * 1000, G_FORMAT_SIZE_BITS), "2.0\302\240Eb");
+  check_string (g_format_size_full (2000ULL, G_FORMAT_SIZE_BITS), "2.0\302\240kbit");
+  check_string (g_format_size_full (2000ULL * 1000, G_FORMAT_SIZE_BITS), "2.0\302\240Mbit");
+  check_string (g_format_size_full (2000ULL * 1000 * 1000, G_FORMAT_SIZE_BITS), "2.0\302\240Gbit");
+  check_string (g_format_size_full (2000ULL * 1000 * 1000 * 1000, G_FORMAT_SIZE_BITS), "2.0\302\240Tbit");
+  check_string (g_format_size_full (2000ULL * 1000 * 1000 * 1000 * 1000, G_FORMAT_SIZE_BITS), "2.0\302\240Pbit");
+  check_string (g_format_size_full (2000ULL * 1000 * 1000 * 1000 * 1000 * 1000, G_FORMAT_SIZE_BITS), "2.0\302\240Ebit");
 
-  check_string (g_format_size_full (238472938, G_FORMAT_SIZE_BITS), "238.5\302\240Mb");
-  check_string (g_format_size_full (238472938, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_LONG_FORMAT), "238.5\302\240Mb (238472938 bits)");
+  check_string (g_format_size_full (238472938, G_FORMAT_SIZE_BITS), "238.5\302\240Mbit");
+  check_string (g_format_size_full (238472938, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_LONG_FORMAT), "238.5\302\240Mbit (238472938 bits)");
   check_string (g_format_size_full (238472938, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_ONLY_VALUE), "238.5");
-  check_string (g_format_size_full (238472938, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_ONLY_UNIT), "Mb");
+  check_string (g_format_size_full (238472938, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_ONLY_UNIT), "Mbit");
 
 
   check_string (g_format_size_full (0, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS), "0 bits");
@@ -1002,17 +1001,17 @@ test_format_size_for_display (void)
   check_string (g_format_size_full (2, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS | G_FORMAT_SIZE_ONLY_VALUE), "2");
   check_string (g_format_size_full (2, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS | G_FORMAT_SIZE_ONLY_UNIT), "bits");
 
-  check_string (g_format_size_full (2048ULL, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS), "2.0\302\240Kib");
-  check_string (g_format_size_full (2048ULL * 1024, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS), "2.0\302\240Mib");
-  check_string (g_format_size_full (2048ULL * 1024 * 1024, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS), "2.0\302\240Gib");
-  check_string (g_format_size_full (2048ULL * 1024 * 1024 * 1024, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS), "2.0\302\240Tib");
-  check_string (g_format_size_full (2048ULL * 1024 * 1024 * 1024 * 1024, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS), "2.0\302\240Pib");
-  check_string (g_format_size_full (2048ULL * 1024 * 1024 * 1024 * 1024 * 1024, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS), "2.0\302\240Eib");
+  check_string (g_format_size_full (2048ULL, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS), "2.0\302\240Kibit");
+  check_string (g_format_size_full (2048ULL * 1024, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS), "2.0\302\240Mibit");
+  check_string (g_format_size_full (2048ULL * 1024 * 1024, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS), "2.0\302\240Gibit");
+  check_string (g_format_size_full (2048ULL * 1024 * 1024 * 1024, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS), "2.0\302\240Tibit");
+  check_string (g_format_size_full (2048ULL * 1024 * 1024 * 1024 * 1024, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS), "2.0\302\240Pibit");
+  check_string (g_format_size_full (2048ULL * 1024 * 1024 * 1024 * 1024 * 1024, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS), "2.0\302\240Eibit");
 
-  check_string (g_format_size_full (238472938, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS), "227.4\302\240Mib");
-  check_string (g_format_size_full (238472938, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS | G_FORMAT_SIZE_LONG_FORMAT), "227.4\302\240Mib (238472938 bits)");
+  check_string (g_format_size_full (238472938, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS), "227.4\302\240Mibit");
+  check_string (g_format_size_full (238472938, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS | G_FORMAT_SIZE_LONG_FORMAT), "227.4\302\240Mibit (238472938 bits)");
   check_string (g_format_size_full (238472938, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS | G_FORMAT_SIZE_ONLY_VALUE), "227.4");
-  check_string (g_format_size_full (238472938, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS | G_FORMAT_SIZE_ONLY_UNIT), "Mib");
+  check_string (g_format_size_full (238472938, G_FORMAT_SIZE_BITS | G_FORMAT_SIZE_IEC_UNITS | G_FORMAT_SIZE_ONLY_UNIT), "Mibit");
 }
 
 static void
@@ -1341,7 +1340,7 @@ test_mkstemp (void)
   g_free (name);
 
   /* Test normal case */
-  name = g_strdup ("testXXXXXXtest"),
+  name = g_build_filename (g_get_tmp_dir (), "testXXXXXXtest", NULL),
   fd = g_mkstemp (name);
   g_assert_cmpint (fd, !=, -1);
   g_assert_null (strstr (name, "XXXXXX"));
@@ -1357,8 +1356,8 @@ test_mkstemp (void)
   strcpy (template, "foobarXXX");
   g_assert_cmpint (g_mkstemp (template), ==, -1);
 
-  strcpy (template, "fooXXXXXX");
-  fd = g_mkstemp (template);
+  name = g_build_filename (g_get_tmp_dir (), "fooXXXXXX", NULL);
+  fd = g_mkstemp (name);
   g_assert_cmpint (fd, !=, -1);
   result = write (fd, hello, hellolen);
   g_assert_cmpint (result, !=, -1);
@@ -1373,15 +1372,16 @@ test_mkstemp (void)
   g_assert_cmpstr (chars, ==, hello);
 
   close (fd);
-  remove (template);
+  remove (name);
+  g_free (name);
 
-  /* Check that is does not work for "fooXXXXXX.pdf" */
-  strcpy (template, "fooXXXXXX.pdf");
-  fd = g_mkstemp (template);
+  /* Check that it works for "fooXXXXXX.pdf" */
+  name = g_build_filename (g_get_tmp_dir (), "fooXXXXXX.pdf", NULL);
+  fd = g_mkstemp (name);
   g_assert_cmpint (fd, !=, -1);
-
   close (fd);
-  remove (template);
+  remove (name);
+  g_free (name);
 }
 
 static void
@@ -1389,12 +1389,12 @@ test_mkdtemp (void)
 {
   gint fd;
   gchar *ret;
-  gchar *name;
+  gchar *name, *name2;
   char template[32];
 
-  name = g_strdup ("testXXXXXXtest"),
+  name = g_build_filename (g_get_tmp_dir (), "testXXXXXXtest", NULL),
   ret = g_mkdtemp (name);
-  g_assert (ret == name);
+  g_assert_true (ret == name);
   g_assert_null (strstr (name, "XXXXXX"));
   g_rmdir (name);
   g_free (name);
@@ -1410,27 +1410,29 @@ test_mkdtemp (void)
   strcpy (template, "foodir");
   g_assert_null (g_mkdtemp (template));
 
-  strcpy (template, "fooXXXXXX");
-  ret = g_mkdtemp (template);
+  name = g_build_filename (g_get_tmp_dir (), "fooXXXXXX", NULL);
+  ret = g_mkdtemp (name);
   g_assert_nonnull (ret);
-  g_assert_true (ret == template);
-  g_assert_false (g_file_test (template, G_FILE_TEST_IS_REGULAR));
-  g_assert_true (g_file_test (template, G_FILE_TEST_IS_DIR));
+  g_assert_true (ret == name);
+  g_assert_false (g_file_test (name, G_FILE_TEST_IS_REGULAR));
+  g_assert_true (g_file_test (name, G_FILE_TEST_IS_DIR));
 
-  strcat (template, "/abc");
-  fd = g_open (template, O_WRONLY | O_CREAT, 0600);
+  name2 = g_build_filename (name, "abc", NULL);
+  fd = g_open (name2, O_WRONLY | O_CREAT, 0600);
   g_assert_cmpint (fd, !=, -1);
   close (fd);
-  g_assert_true (g_file_test (template, G_FILE_TEST_IS_REGULAR));
-  g_assert_cmpint (g_unlink (template), !=, -1);
+  g_assert_true (g_file_test (name2, G_FILE_TEST_IS_REGULAR));
+  g_assert_cmpint (g_unlink (name2), !=, -1);
+  g_free (name2);
 
-  template[9] = '\0';
-  g_assert_cmpint (g_rmdir (template), !=, -1);
+  g_assert_cmpint (g_rmdir (name), !=, -1);
+  g_free (name);
 
-  strcpy (template, "fooXXXXXX.dir");
-  g_assert_nonnull (g_mkdtemp (template));
-  g_assert_true (g_file_test (template, G_FILE_TEST_IS_DIR));
-  g_rmdir (template);
+  name = g_build_filename (g_get_tmp_dir (), "fooXXXXXX.dir", NULL);
+  g_assert_nonnull (g_mkdtemp (name));
+  g_assert_true (g_file_test (name, G_FILE_TEST_IS_DIR));
+  g_rmdir (name);
+  g_free (name);
 }
 
 static void
@@ -1441,7 +1443,7 @@ test_get_contents (void)
   gchar *contents;
   GError *error = NULL;
   const gchar *text = "abcdefghijklmnopqrstuvwxyz";
-  const gchar *filename = "file-test-get-contents";
+  char *filename = g_build_filename (g_get_tmp_dir (), "file-test-get-contents", NULL);
   gsize bytes_written;
 
   f = g_fopen (filename, "w");
@@ -1470,6 +1472,125 @@ test_get_contents (void)
 
   g_free (contents);
   g_remove (filename);
+  g_free (filename);
+}
+
+static gboolean
+resize_file (const gchar *filename,
+             gint64       size)
+{
+  int fd;
+  int retval;
+
+  fd = g_open (filename, O_CREAT | O_RDWR | O_TRUNC, 0666);
+  g_assert_cmpint (fd, >=, 0);
+
+#ifdef G_OS_WIN32
+  retval = _chsize_s (fd, size);
+#elif HAVE_FTRUNCATE64
+  retval = ftruncate64 (fd, size);
+#else
+  errno = ENOSYS;
+  retval = -1;
+#endif
+  if (retval != 0)
+    {
+      g_test_message ("Error trying to resize file (%s)", strerror (errno));
+      close (fd);
+      return FALSE;
+    }
+
+  close (fd);
+  return TRUE;
+}
+
+static gboolean
+is_error_in_list (GFileError       error_code,
+                  const GFileError ok_list[],
+                  size_t           ok_count)
+{
+  for (size_t i = 0; i < ok_count; i++)
+    {
+      if (ok_list[i] == error_code)
+        return TRUE;
+    }
+  return FALSE;
+}
+
+static void
+get_largefile_check_len (const gchar      *filename,
+                         gint64            large_len,
+                         const GFileError  ok_list[],
+                         size_t            ok_count)
+{
+  gboolean get_ok;
+  gsize len;
+  gchar *contents;
+  GError *error = NULL;
+
+  get_ok = g_file_get_contents (filename, &contents, &len, &error);
+  if (get_ok)
+    {
+      g_assert_cmpint ((gint64) len, ==, large_len);
+      g_free (contents);
+    }
+  else
+    {
+      g_assert_cmpint (error->domain, ==, G_FILE_ERROR);
+      if (is_error_in_list ((GFileError)error->code, ok_list, ok_count))
+        {
+          g_test_message ("Error reading file of size 0x%" G_GINT64_MODIFIER "x, but with acceptable error type (%s)", large_len, error->message);
+        }
+      else
+        {
+          /* fail for other errors */
+          g_assert_no_error (error);
+        }
+      g_clear_error (&error);
+    }
+}
+
+static void
+test_get_contents_largefile (void)
+{
+  if (!g_test_slow ())
+    {
+      g_test_skip ("Skipping slow largefile test");
+      return;
+    }
+
+  const gchar *filename = "file-test-get-contents-large";
+  gint64 large_len;
+
+  /* error OK if couldn't allocate large buffer, or if file is too large */
+  const GFileError too_large_errors[] = { G_FILE_ERROR_NOMEM, G_FILE_ERROR_FAILED };
+  /* error OK if couldn't allocate large buffer */
+  const GFileError nomem_errors[] = { G_FILE_ERROR_NOMEM };
+
+  /* OK to fail to read this, but don't silently under-read */
+  large_len = (G_GINT64_CONSTANT (1) << 32) + 16;
+  if (!resize_file (filename, large_len))
+    goto failed_resize;
+  get_largefile_check_len (filename, large_len, too_large_errors, G_N_ELEMENTS (too_large_errors));
+
+  /* OK to fail to read this size, but don't silently under-read */
+  large_len = (G_GINT64_CONSTANT (1) << 32) - 1;
+  if (!resize_file (filename, large_len))
+    goto failed_resize;
+  get_largefile_check_len (filename, large_len, too_large_errors, G_N_ELEMENTS (too_large_errors));
+
+  /* OK to fail memory allocation, but don't otherwise fail this size */
+  large_len = (G_GINT64_CONSTANT (1) << 31) - 1;
+  if (!resize_file (filename, large_len))
+    goto failed_resize;
+  get_largefile_check_len (filename, large_len, nomem_errors, G_N_ELEMENTS (nomem_errors));
+
+  g_remove (filename);
+  return;
+
+failed_resize:
+  g_test_incomplete ("Failed to resize large file, unable to complete large file tests.");
+  g_remove (filename);
 }
 
 static void
@@ -1973,7 +2094,7 @@ test_stdio_wrappers (void)
 
   g_remove ("mkdir-test/test-create");
   ret = g_rmdir ("mkdir-test");
-  g_assert (ret == 0 || errno == ENOENT);
+  g_assert_true (ret == 0 || errno == ENOENT);
 
   ret = g_stat ("mkdir-test", &buf);
   g_assert_cmpint (ret, ==, -1);
@@ -2074,7 +2195,7 @@ test_stdio_wrappers (void)
 static void
 test_fopen_modes (void)
 {
-  char        *path = g_build_filename ("temp-fopen", NULL);
+  char        *path = g_build_filename (g_get_tmp_dir (), "temp-fopen", NULL);
   gsize        i;
   const gchar *modes[] =
     {
@@ -2475,26 +2596,6 @@ test_win32_zero_terminate_symlink (void)
 
 #endif
 
-static void
-assert_fd_was_closed (int fd)
-{
-  /* We can't tell a fd was really closed without behaving as though it
-   * was still valid */
-  if (g_test_undefined ())
-    {
-      int result, errsv;
-      GWin32InvalidParameterHandler handler;
-
-      GLIB_PRIVATE_CALL (g_win32_push_empty_invalid_parameter_handler) (&handler);
-      result = g_fsync (fd);
-      errsv = errno;
-      GLIB_PRIVATE_CALL (g_win32_pop_invalid_parameter_handler) (&handler);
-
-      g_assert_cmpint (result, !=, 0);
-      g_assert_cmpint (errsv, ==, EBADF);
-    }
-}
-
 static void
 test_clear_fd_ebadf (void)
 {
@@ -2666,6 +2767,7 @@ main (int   argc,
   g_test_add_func ("/fileutils/mkstemp", test_mkstemp);
   g_test_add_func ("/fileutils/mkdtemp", test_mkdtemp);
   g_test_add_func ("/fileutils/get-contents", test_get_contents);
+  g_test_add_func ("/fileutils/get-contents-large-file", test_get_contents_largefile);
   g_test_add_func ("/fileutils/set-contents", test_set_contents);
   g_test_add_func ("/fileutils/set-contents-full", test_set_contents_full);
   g_test_add_func ("/fileutils/set-contents-full/read-only-file", test_set_contents_full_read_only_file);
