@@ -2,6 +2,8 @@
  * Copyright (C) 2007 Openismus GmbH
  * Authors: Mathias Hasselmann
  *
+ * SPDX-License-Identifier: LicenseRef-old-glib-tests
+ *
  * This work is provided "as is"; redistribution and modification
  * in whole or in part, in any medium, physical or electronic is
  * permitted without restriction.
@@ -28,15 +30,14 @@
 #include <locale.h>
 #include <math.h>
 
-
-static GOptionEntry main_entries[] = {
+static GOptionEntry global_main_entries[] = {
   { "main-switch", 0, 0,
     G_OPTION_ARG_NONE, NULL,
     "A switch that is in the main group", NULL },
   G_OPTION_ENTRY_NULL
 };
 
-static GOptionEntry group_entries[] = {
+static GOptionEntry global_group_entries[] = {
   { "test-switch", 0, 0,
     G_OPTION_ARG_NONE, NULL,
     "A switch that is in the test group", NULL },
@@ -54,14 +55,14 @@ make_options (int test_number)
   options = g_option_context_new (NULL);
 
   if (have_main_entries)
-    g_option_context_add_main_entries (options, main_entries, NULL);
+    g_option_context_add_main_entries (options, global_main_entries, NULL);
   if (have_test_entries)
     {
       group = g_option_group_new ("test", "Test Options",
                                   "Show all test options",
                                   NULL, NULL);
       g_option_context_add_group (options, group);
-      g_option_group_add_entries (group, group_entries);
+      g_option_group_add_entries (group, global_group_entries);
     }
 
   return options;
@@ -2210,6 +2211,31 @@ test_help_no_help_options (void)
   g_option_context_free (context);
 }
 
+static void
+test_help_deprecated (void)
+{
+  GOptionContext *context;
+  gchar *str;
+  gchar *arg = NULL;
+  GOptionEntry entries[] = {
+    { "test", 't', G_OPTION_FLAG_DEPRECATED, G_OPTION_ARG_STRING, &arg, "Test tests", "Argument to use in test" },
+    { "test2", 0, 0, G_OPTION_ARG_NONE, NULL, "Tests also", NULL },
+    G_OPTION_ENTRY_NULL
+  };
+
+  context = g_option_context_new ("blabla");
+  g_option_context_add_main_entries (context, entries, NULL);
+  g_option_context_set_summary (context, "Summary");
+  g_option_context_set_description (context, "Description");
+
+  str = g_option_context_get_help (context, FALSE, NULL);
+  g_test_message ("%s", str);
+  g_assert (strstr (str, "(deprecated)") != NULL);
+  g_free (str);
+
+  g_option_context_free (context);
+}
+
 static void
 set_bool (gpointer data)
 {
@@ -2607,6 +2633,7 @@ main (int   argc,
   g_test_add_func ("/option/help/options", test_help);
   g_test_add_func ("/option/help/no-options", test_help_no_options);
   g_test_add_func ("/option/help/no-help-options", test_help_no_help_options);
+  g_test_add_func ("/option/help/deprecated", test_help_deprecated);
 
   g_test_add_func ("/option/basic", test_basic);
   g_test_add_func ("/option/translate", test_translate);
