@@ -40,7 +40,7 @@ test_sort_basic (void)
       data[i] = g_random_int_range (0, 10000);
     }
 
-  g_qsort_with_data (data, 10000, sizeof (int), int_compare_data, NULL);
+  g_sort_array (data, 10000, sizeof (int), int_compare_data, NULL);
 
   for (i = 1; i < 10000; i++)
     g_assert_cmpint (data[i -1], <=, data[i]);
@@ -63,7 +63,7 @@ test_sort_zero_elements (void)
     }
 
   /* 0 elements is a valid case */
-  g_qsort_with_data (data, 0, sizeof (int), int_compare_data, NULL);
+  g_sort_array (data, 0, sizeof (int), int_compare_data, NULL);
 
   for (i = 0; i < 100; i++)
     g_assert_cmpint (data[i], ==, data_copy[i]);
@@ -105,7 +105,7 @@ test_sort_stable (void)
       data[i].i = i;
     }
 
-  g_qsort_with_data (data, 10000, sizeof (SortItem), item_compare_data, NULL);
+  g_sort_array (data, 10000, sizeof (SortItem), item_compare_data, NULL);
 
   for (i = 1; i < 10000; i++)
     {
@@ -129,7 +129,7 @@ test_sort_big (void)
       data[i].i = i;
     }
 
-  g_qsort_with_data (data, 10000, sizeof (BigItem), item_compare_data, NULL);
+  g_sort_array (data, 10000, sizeof (BigItem), item_compare_data, NULL);
 
   for (i = 1; i < 10000; i++)
     {
@@ -140,6 +140,28 @@ test_sort_big (void)
   g_free (data);
 }
 
+static void
+test_sort_deprecated (void)
+{
+  gint *data;
+  gint i;
+
+  data = g_malloc (10000 * sizeof (int));
+  for (i = 0; i < 10000; i++)
+    {
+      data[i] = g_random_int_range (0, 10000);
+    }
+
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+  g_qsort_with_data (data, 10000, sizeof (int), int_compare_data, NULL);
+G_GNUC_END_IGNORE_DEPRECATIONS
+
+  for (i = 1; i < 10000; i++)
+    g_assert_cmpint (data[i -1], <=, data[i]);
+
+  g_free (data);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -149,6 +171,7 @@ main (int argc, char *argv[])
   g_test_add_func ("/sort/zero-elements", test_sort_zero_elements);
   g_test_add_func ("/sort/stable", test_sort_stable);
   g_test_add_func ("/sort/big", test_sort_big);
+  g_test_add_func ("/sort/deprecated", test_sort_deprecated);
 
   return g_test_run ();
 }
