@@ -173,6 +173,11 @@ test_unichar_break_type (void)
     { G_UNICODE_BREAK_EMOJI_BASE,          0x1F466 },
     { G_UNICODE_BREAK_EMOJI_MODIFIER,      0x1F3FB },
     { G_UNICODE_BREAK_ZERO_WIDTH_JOINER,   0x200D },
+    { G_UNICODE_BREAK_AKSARA,              0x1B45 },
+    { G_UNICODE_BREAK_AKSARA_PRE_BASE,     0x1193F },
+    { G_UNICODE_BREAK_AKSARA_START,        0x11F50 },
+    { G_UNICODE_BREAK_VIRAMA_FINAL,        0x1BF3 },
+    { G_UNICODE_BREAK_VIRAMA,              0xA9C0 },
   };
 
   for (i = 0; i < G_N_ELEMENTS (examples); i++)
@@ -359,6 +364,13 @@ test_unichar_script (void)
     { G_UNICODE_SCRIPT_VITHKUQI,               0x10570 },
     { G_UNICODE_SCRIPT_KAWI,                   0x11F00 },
     { G_UNICODE_SCRIPT_NAG_MUNDARI,            0x1E4D0 },
+    { G_UNICODE_SCRIPT_TODHRI,                 0x105C8 },
+    { G_UNICODE_SCRIPT_GARAY,                  0x10D40 },
+    { G_UNICODE_SCRIPT_TULU_TIGALARI,          0x11387 },
+    { G_UNICODE_SCRIPT_SUNUWAR,                0x11BC0 },
+    { G_UNICODE_SCRIPT_GURUNG_KHEMA,           0x16139 },
+    { G_UNICODE_SCRIPT_KIRAT_RAI,              0x16D40 },
+    { G_UNICODE_SCRIPT_OL_ONAL,                0x1E5D0 },
   };
   for (i = 0; i < G_N_ELEMENTS (examples); i++)
     g_assert_cmpint (g_unichar_get_script (examples[i].c), ==, examples[i].script);
@@ -406,7 +418,15 @@ test_combining_class (void)
     { 233, 0x0362 },
     { 234, 0x0360 },
     { 234, 0x1DCD },
-    { 240, 0x0345 }
+    { 240, 0x0345 },
+    /* These are all (currently) unassigned, but exercise various branches in
+     * the combining class lookup tables: */
+    { 0, 0x323FF },
+    { 0, 0x32400 },
+    { 0, 0xDFFFF },
+    { 0, 0xE0000 },
+    { 0, G_UNICODE_LAST_CHAR },
+    { 0, G_UNICODE_LAST_CHAR + 1 },
   };
   for (i = 0; i < G_N_ELEMENTS (examples); i++)
     {
@@ -459,6 +479,10 @@ test_strup (void)
   /* Tricky, comparing two unicode strings with an ASCII function */
   g_assert_cmpstr (str_up, ==, "AAZZ09X;\003E\357\274\241\357\274\241");
   g_free (str_up);
+
+  str_up = g_utf8_strup ("", 0);
+  g_assert_cmpstr (str_up, ==, "");
+  g_free (str_up);
 }
 
 /* Test that g_utf8_strdown() returns the correct value for various
@@ -484,6 +508,10 @@ test_strdown (void)
   /* Tricky, comparing two unicode strings with an ASCII function */
   g_assert_cmpstr (str_down, ==, "aazz09x;\003\007\357\275\201\357\275\201");
   g_free (str_down);
+
+  str_down = g_utf8_strdown ("", 0);
+  g_assert_cmpstr (str_down, ==, "");
+  g_free (str_down);
 }
 
 /* Test that g_utf8_strup() and g_utf8_strdown() return the correct
@@ -512,6 +540,7 @@ test_turkish_strupdown (void)
   if (oldlocale == NULL)
     {
       g_test_skip ("locale tr_TR not available");
+      g_free (old_lang);
       return;
     }
 
@@ -576,6 +605,10 @@ test_casefold (void)
   /* Tricky, comparing two unicode strings with an ASCII function */
   g_assert_cmpstr (str_casefold, ==, "aazz09x;\357\275\201\357\275\201");
   g_free (str_casefold);
+
+  str_casefold = g_utf8_casefold ("", 0);
+  g_assert_cmpstr (str_casefold, ==, "");
+  g_free (str_casefold);
 }
 
 static void
@@ -1397,36 +1430,102 @@ test_wide (void)
 static void
 test_compose (void)
 {
-  gunichar ch;
+  const struct
+    {
+      gunichar a;
+      gunichar b;
+      gunichar expected_result;  /* 0 for failure */
+    }
+  vectors[] =
+    {
+      /* Not composable */
+      { 0x0041, 0x0042, 0 },
+      { 0x0041, 0x0000, 0 },
+      { 0x0066, 0x0069, 0 },
 
-  /* Not composable */
-  g_assert_false (g_unichar_compose (0x0041, 0x0042, &ch) && ch == 0);
-  g_assert_false (g_unichar_compose (0x0041, 0, &ch) && ch == 0);
-  g_assert_false (g_unichar_compose (0x0066, 0x0069, &ch) && ch == 0);
+      /* Tricky non-composable */
+      { 0x0308, 0x0301, 0 }, /* !0x0344 */
+      { 0x0F71, 0x0F72, 0 }, /* !0x0F73 */
 
-  /* Tricky non-composable */
-  g_assert_false (g_unichar_compose (0x0308, 0x0301, &ch) && ch == 0); /* !0x0344 */
-  g_assert_false (g_unichar_compose (0x0F71, 0x0F72, &ch) && ch == 0); /* !0x0F73 */
+      /* Singletons should not compose */
+      { 0x212B, 0x0000, 0 },
+      { 0x00C5, 0x0000, 0 },
+      { 0x2126, 0x0000, 0 },
+      { 0x03A9, 0x0000, 0 },
 
-  /* Singletons should not compose */
-  g_assert_false (g_unichar_compose (0x212B, 0, &ch) && ch == 0);
-  g_assert_false (g_unichar_compose (0x00C5, 0, &ch) && ch == 0);
-  g_assert_false (g_unichar_compose (0x2126, 0, &ch) && ch == 0);
-  g_assert_false (g_unichar_compose (0x03A9, 0, &ch) && ch == 0);
+      /* Pairs */
+      { 0x0041, 0x030A, 0x00C5 },
+      { 0x006F, 0x0302, 0x00F4 },
+      { 0x1E63, 0x0307, 0x1E69 },
+      { 0x0073, 0x0323, 0x1E63 },
+      { 0x0064, 0x0307, 0x1E0B },
+      { 0x0064, 0x0323, 0x1E0D },
 
-  /* Pairs */
-  g_assert_true (g_unichar_compose (0x0041, 0x030A, &ch) && ch == 0x00C5);
-  g_assert_true (g_unichar_compose (0x006F, 0x0302, &ch) && ch == 0x00F4);
-  g_assert_true (g_unichar_compose (0x1E63, 0x0307, &ch) && ch == 0x1E69);
-  g_assert_true (g_unichar_compose (0x0073, 0x0323, &ch) && ch == 0x1E63);
-  g_assert_true (g_unichar_compose (0x0064, 0x0307, &ch) && ch == 0x1E0B);
-  g_assert_true (g_unichar_compose (0x0064, 0x0323, &ch) && ch == 0x1E0D);
+       /* Hangul */
+      { 0xD4CC, 0x11B6, 0xD4DB },
+      { 0x1111, 0x1171, 0xD4CC },
+      { 0xCE20, 0x11B8, 0xCE31 },
+      { 0x110E, 0x1173, 0xCE20 },
 
-  /* Hangul */
-  g_assert_true (g_unichar_compose (0xD4CC, 0x11B6, &ch) && ch == 0xD4DB);
-  g_assert_true (g_unichar_compose (0x1111, 0x1171, &ch) && ch == 0xD4CC);
-  g_assert_true (g_unichar_compose (0xCE20, 0x11B8, &ch) && ch == 0xCE31);
-  g_assert_true (g_unichar_compose (0x110E, 0x1173, &ch) && ch == 0xCE20);
+      /* Hangul non-compositions (testing various exit conditions in combine_hangul()) */
+      { 0x1100, 0x1160, 0 },
+      { 0x1100, 0x1177, 0 },
+      { 0xABFF, 0x11B6, 0 },
+      { 0xD7A5, 0x11B6, 0 },
+      { 0xAC01, 0x11B6, 0 },
+      { 0xD4CC, 0x11A6, 0 },
+      { 0xD4CC, 0x11C4, 0 },
+
+      /* Primary composite above U+FFFF (a significant boundary value in our implementation) */
+      { 0x1611E, 0x1611E, 0x16121 },  /* first and second char equal */
+      { 0x1611E, 0x1611F, 0x16123 },
+
+      /* First singletons */
+      { 0x00F6, 0x0304, 0x022B },
+
+      /* Second singletons */
+      { 0x0B47, 0x0B57, 0x0B4C },
+      { 0x00A0, 0x0B57, 0 },
+
+      /* Very high values (exercising some branches in COMPOSE_INDEX) */
+      { 0x16E00, 0x030A, 0 },
+      { 0x212B, 0x16E00, 0 },
+
+      /* Exercise some failure paths in the lookup tables */
+      { 0x1E63, 0x0306, 0 },
+      { 0x1E63, 0x0304, 0 },
+      { 0x1E63, 0x0B57, 0 },
+      { 0x1E63, 0x0000, 0 },
+      { 0x1E63, 0x113C2, 0 },
+      { 0x1F01, 0x113C2, 0 },
+      { 0x006E, 0x0302, 0 },
+      { 0x1E63, 0x1611F, 0 },
+      { 0x1138E, 0x113B8, 0 },
+      { 0x1611E, 0x0000, 0 },
+      { 0x0000, 0x1611F, 0 },
+      { 0x11390, 0x113C2, 0 },
+    };
+
+  for (size_t i = 0; i < G_N_ELEMENTS (vectors); i++)
+    {
+      gunichar ch;
+      gboolean result;
+
+      g_test_message ("Composing U+%06x and U+%06x; expecting U+%06x",
+                      vectors[i].a, vectors[i].b, vectors[i].expected_result);
+
+      result = g_unichar_compose (vectors[i].a, vectors[i].b, &ch);
+      if (vectors[i].expected_result != 0)
+        {
+          g_assert_cmpuint (ch, ==, vectors[i].expected_result);
+          g_assert_true (result);
+        }
+      else
+        {
+          g_assert_cmpuint (ch, ==, 0);
+          g_assert_false (result);
+        }
+    }
 }
 
 /* Test that g_unichar_decompose() returns the correct value for various
@@ -1461,6 +1560,10 @@ test_decompose (void)
   g_assert_true (g_unichar_decompose (0xD4CC, &a, &b) && a == 0x1111 && b == 0x1171);
   g_assert_true (g_unichar_decompose (0xCE31, &a, &b) && a == 0xCE20 && b == 0x11B8);
   g_assert_true (g_unichar_decompose (0xCE20, &a, &b) && a == 0x110E && b == 0x1173);
+
+  /* Primary composite above U+FFFF (a significant boundary value in our implementation) */
+  g_assert_true (g_unichar_decompose (0x16121, &a, &b) && a == 0x1611E && b == 0x1611E);  /* first and second char equal */
+  g_assert_true (g_unichar_decompose (0x16123, &a, &b) && a == 0x1611E && b == 0x1611F);
 }
 
 /* Test that g_unichar_fully_decompose() returns the correct value for
@@ -1468,50 +1571,76 @@ test_decompose (void)
 static void
 test_fully_decompose_canonical (void)
 {
-  gunichar decomp[5];
-  gsize len;
+  const struct
+    {
+      gunichar input;
+      size_t expected_len;
+      gunichar expected_decomposition[4];
+    }
+  vectors[] =
+    {
+#define TEST0(ch)		{ ch, 1, { ch, 0, 0, 0 }}
+#define TEST1(ch, a)		{ ch, 1, { a, 0, 0, 0 }}
+#define TEST2(ch, a, b)		{ ch, 2, { a, b, 0, 0 }}
+#define TEST3(ch, a, b, c)	{ ch, 3, { a, b, c, 0 }}
+#define TEST4(ch, a, b, c, d)	{ ch, 4, { a, b, c, d }}
 
-#define TEST_DECOMP(ch, expected_len, a, b, c, d) \
-  len = g_unichar_fully_decompose (ch, FALSE, decomp, G_N_ELEMENTS (decomp)); \
-  g_assert_cmpint (expected_len, ==, len); \
-  if (expected_len >= 1) g_assert_cmphex (decomp[0], ==, a); \
-  if (expected_len >= 2) g_assert_cmphex (decomp[1], ==, b); \
-  if (expected_len >= 3) g_assert_cmphex (decomp[2], ==, c); \
-  if (expected_len >= 4) g_assert_cmphex (decomp[3], ==, d); \
+      /* Not decomposable */
+      TEST0 (0x0041),
+      TEST0 (0xFB01),
 
-#define TEST0(ch)		TEST_DECOMP (ch, 1, ch, 0, 0, 0)
-#define TEST1(ch, a)		TEST_DECOMP (ch, 1, a, 0, 0, 0)
-#define TEST2(ch, a, b)		TEST_DECOMP (ch, 2, a, b, 0, 0)
-#define TEST3(ch, a, b, c)	TEST_DECOMP (ch, 3, a, b, c, 0)
-#define TEST4(ch, a, b, c, d)	TEST_DECOMP (ch, 4, a, b, c, d)
+      /* Singletons */
+      TEST2 (0x212B, 0x0041, 0x030A),
+      TEST1 (0x2126, 0x03A9),
 
-  /* Not decomposable */
-  TEST0 (0x0041);
-  TEST0 (0xFB01);
+      /* Tricky pairs */
+      TEST2 (0x0344, 0x0308, 0x0301),
+      TEST2 (0x0F73, 0x0F71, 0x0F72),
 
-  /* Singletons */
-  TEST2 (0x212B, 0x0041, 0x030A);
-  TEST1 (0x2126, 0x03A9);
+      /* General */
+      TEST2 (0x00C5, 0x0041, 0x030A),
+      TEST2 (0x00F4, 0x006F, 0x0302),
+      TEST3 (0x1E69, 0x0073, 0x0323, 0x0307),
+      TEST2 (0x1E63, 0x0073, 0x0323),
+      TEST2 (0x1E0B, 0x0064, 0x0307),
+      TEST2 (0x1E0D, 0x0064, 0x0323),
 
-  /* Tricky pairs */
-  TEST2 (0x0344, 0x0308, 0x0301);
-  TEST2 (0x0F73, 0x0F71, 0x0F72);
+      /* Hangul */
+      TEST3 (0xD4DB, 0x1111, 0x1171, 0x11B6),
+      TEST2 (0xD4CC, 0x1111, 0x1171),
+      TEST3 (0xCE31, 0x110E, 0x1173, 0x11B8),
+      TEST2 (0xCE20, 0x110E, 0x1173),
 
-  /* General */
-  TEST2 (0x00C5, 0x0041, 0x030A);
-  TEST2 (0x00F4, 0x006F, 0x0302);
-  TEST3 (0x1E69, 0x0073, 0x0323, 0x0307);
-  TEST2 (0x1E63, 0x0073, 0x0323);
-  TEST2 (0x1E0B, 0x0064, 0x0307);
-  TEST2 (0x1E0D, 0x0064, 0x0323);
+#undef TEST4
+#undef TEST3
+#undef TEST2
+#undef TEST1
+#undef TEST0
+    };
 
-  /* Hangul */
-  TEST3 (0xD4DB, 0x1111, 0x1171, 0x11B6);
-  TEST2 (0xD4CC, 0x1111, 0x1171);
-  TEST3 (0xCE31, 0x110E, 0x1173, 0x11B8);
-  TEST2 (0xCE20, 0x110E, 0x1173);
+  for (size_t i = 0; i < G_N_ELEMENTS (vectors); i++)
+    {
+      gunichar decomp[5];
+      size_t len;
 
-#undef TEST_DECOMP
+      g_test_message ("Fully decomposing U+%06x; expecting %" G_GSIZE_FORMAT " codepoints",
+                      vectors[i].input, vectors[i].expected_len);
+
+      /* Test with all possible output array sizes, to check that the function
+       * can write partial results OK. */
+      for (size_t j = 0; j <= G_N_ELEMENTS (decomp); j++)
+        {
+          len = g_unichar_fully_decompose (vectors[i].input, FALSE, decomp, G_N_ELEMENTS (decomp) - j);
+          g_assert_cmpuint (len, ==, vectors[i].expected_len);
+          if (len >= j)
+            g_assert_cmpmem (decomp, (len - j) * sizeof (*decomp),
+                             vectors[i].expected_decomposition, (vectors[i].expected_len - j) * sizeof (*vectors[i].expected_decomposition));
+        }
+
+      /* And again with no result array at all, just to get the length. */
+      len = g_unichar_fully_decompose (vectors[i].input, FALSE, NULL, 0);
+      g_assert_cmpuint (len, ==, vectors[i].expected_len);
+    }
 }
 
 /* Test that g_unicode_canonical_decomposition() returns the correct
@@ -1857,6 +1986,15 @@ test_iso15924 (void)
     /* Unicode 15.0 additions */
     { G_UNICODE_SCRIPT_KAWI,                   "Kawi" },
     { G_UNICODE_SCRIPT_NAG_MUNDARI,            "Nagm" },
+
+    /* Unicode 16.0 additions */
+    { G_UNICODE_SCRIPT_TODHRI,                 "Todr" },
+    { G_UNICODE_SCRIPT_GARAY,                  "Gara" },
+    { G_UNICODE_SCRIPT_TULU_TIGALARI,          "Tutg" },
+    { G_UNICODE_SCRIPT_SUNUWAR,                "Sunu" },
+    { G_UNICODE_SCRIPT_GURUNG_KHEMA,           "Gukh" },
+    { G_UNICODE_SCRIPT_KIRAT_RAI,              "Krai" },
+    { G_UNICODE_SCRIPT_OL_ONAL,                "Onao" },
   };
   guint i;
 
@@ -1889,6 +2027,45 @@ test_iso15924 (void)
 #undef PACK
 }
 
+static void
+test_normalize (void)
+{
+  guint i;
+  typedef struct
+  {
+    const gchar *str;
+    const gchar *nfd;
+    const gchar *nfc;
+    const gchar *nfkd;
+    const gchar *nfkc;
+  } Test;
+  Test tests[] = {
+    { "Äffin", "A\u0308ffin", "Äffin", "A\u0308ffin", "Äffin" },
+    { "Ä\uFB03n", "A\u0308\uFB03n", "Ä\uFB03n", "A\u0308ffin", "Äffin" },
+    { "Henry IV", "Henry IV", "Henry IV", "Henry IV", "Henry IV" },
+    { "Henry \u2163", "Henry \u2163", "Henry \u2163", "Henry IV", "Henry IV" },
+    { "non-utf\x88", NULL, NULL, NULL, NULL },
+    { "", "", "", "", "" },
+  };
+
+#define TEST(str, mode, expected)                         \
+  {                                                       \
+    gchar *normalized = g_utf8_normalize (str, -1, mode); \
+    g_assert_cmpstr (normalized, ==, expected);           \
+    g_free (normalized);                                  \
+  }
+
+  for (i = 0; i < G_N_ELEMENTS (tests); i++)
+    {
+      TEST (tests[i].str, G_NORMALIZE_NFD, tests[i].nfd);
+      TEST (tests[i].str, G_NORMALIZE_NFC, tests[i].nfc);
+      TEST (tests[i].str, G_NORMALIZE_NFKD, tests[i].nfkd);
+      TEST (tests[i].str, G_NORMALIZE_NFKC, tests[i].nfkc);
+    }
+
+#undef TEST
+}
+
 int
 main (int   argc,
       char *argv[])
@@ -1933,6 +2110,7 @@ main (int   argc,
   g_test_add_func ("/unicode/xdigit", test_xdigit);
   g_test_add_func ("/unicode/xdigit-value", test_xdigit_value);
   g_test_add_func ("/unicode/zero-width", test_zerowidth);
+  g_test_add_func ("/unicode/normalize", test_normalize);
 
   return g_test_run();
 }
